#include "core/Components.h"

void coreInitComponents(CoreComponents* c, size_t componentSize) {
    coreInitHashMap(&c->entityToIndex, sizeof(CoreEntity), sizeof(size_t));
    c->indexToEntity = CORE_LIST(sizeof(CoreEntity));
    c->components = CORE_LIST(componentSize);
}

void coreDestroyComponents(CoreComponents* c) {
    coreDestroyHashMap(&c->entityToIndex);
    coreDestroyList(&c->indexToEntity);
    coreDestroyList(&c->components);
}

void* coreComponentsGetOrAdd(CoreComponents* c, CoreEntity e) {
    void* component = coreComponentsSearch(c, e);
    if(component != nullptr) {
        return component;
    }
    size_t index = c->components.length;
    coreHashMapPutPointer(&c->entityToIndex, &e, &index);
    coreListAddPointer(&c->indexToEntity, &e);
    return coreListAddEmpty(&c->components);
}

void* coreComponentsSearch(CoreComponents* c, CoreEntity e) {
    size_t* index = coreHashMapSearchPointer(&c->entityToIndex, &e);
    if(index == nullptr) {
        return nullptr;
    }
    return coreListGetVoidPointer(&c->components, *index);
}

bool coreComponentsRemove(CoreComponents* c, CoreEntity e) {
    size_t* indexP = coreHashMapSearchPointer(&c->entityToIndex, &e);
    if(indexP == nullptr) {
        return false;
    }
    size_t lastIndex = c->components.length - 1;
    size_t index = *indexP;
    coreHashMapRemovePointer(&c->entityToIndex, &e);
    coreListRemoveBySwap(&c->components, index);
    if(index == lastIndex) {
        coreListRemoveBySwap(&c->indexToEntity, index);
        return true;
    }
    CoreEntity other = coreListGet(&c->indexToEntity, lastIndex, CoreEntity);
    coreListRemoveBySwap(&c->indexToEntity, index);
    coreHashMapPutPointer(&c->entityToIndex, &other, &index);
    return true;
}

void coreInitComponentsIterator(CoreComponentIterator* ci, CoreComponents* c) {
    ci->indexToEntity = coreListBegin(&c->indexToEntity);
    ci->indexToEntityEnd = coreListEnd(&c->indexToEntity);
    ci->component = coreListBegin(&c->components);
    ci->componentEnd = coreListEnd(&c->components);
    ci->componentSize = c->components.dataSize;
    ci->node = (CoreComponentNode){0};
}

bool coreComponentsHasNext(CoreComponentIterator* ci) {
    return ci->indexToEntity != ci->indexToEntityEnd;
}

CoreComponentNode* coreComponentsNext(CoreComponentIterator* ci) {
    ci->node.component = ci->component;
    ci->node.entity = *ci->indexToEntity;
    ci->indexToEntity++;
    ci->component = (char*)ci->component + ci->componentSize;
    return &ci->node;
}

void* coreComponentsBegin(CoreComponents* c) {
    return coreListBegin(&c->components);
}

void* coreComponentsEnd(CoreComponents* c) {
    return coreListEnd(&c->components);
}
